/* (C) 2022 YiRing, Inc. */
package com.yiring.app.config.zy;

import javax.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.connection.CachingConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.amqp.SimpleRabbitListenerContainerFactoryConfigurer;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 真源 RabbitMQ 订阅配置
 *
 * @author Jim
 * @version 0.1
 * 2022/4/13 17:05
 */

@Slf4j
@Configuration
public class ZyRabbitConfig {

    @Resource
    ZyConfigProperties.ZyConfigRabbitmq rabbitmq;

    @Resource
    ConfigurableApplicationContext context;

    public static final String TEMPLATE_NAME = "zyRabbitTemplate";
    public static final String CONNECTION_FACTORY_NAME = "zyRabbitConnectionFactory";
    public static final String LISTENER_FACTORY_NAME = "zyRabbitListenerFactory";

    /**
     * 消息队列名称（必须要与配置文件中的 queue-name 完全一致）
     * 规则: tenant_msg_${open.client-secret}_${open.client-id}
     * 参见文档: 定位平台接口规范V3.0.1 - 通用版.pdf #6
     */
    public static final String MESSAGE_QUEUES_NAME = "tenant_msg_12A14FDC_sc21080400";

    @Bean(name = CONNECTION_FACTORY_NAME)
    public CachingConnectionFactory zyConnectionFactory() {
        return connectionFactory(
            rabbitmq.getHost(),
            rabbitmq.getPort(),
            rabbitmq.getUsername(),
            rabbitmq.getPassword(),
            rabbitmq.getVirtualHost()
        );
    }

    @Bean(name = TEMPLATE_NAME)
    public RabbitTemplate secondRabbitTemplate(
        @Qualifier(CONNECTION_FACTORY_NAME) CachingConnectionFactory connectionFactory
    ) {
        return new RabbitTemplate(connectionFactory);
    }

    @Bean(name = LISTENER_FACTORY_NAME)
    public SimpleRabbitListenerContainerFactory secondFactory(
        SimpleRabbitListenerContainerFactoryConfigurer configurer,
        @Qualifier(CONNECTION_FACTORY_NAME) CachingConnectionFactory connectionFactory
    ) {
        // 检查队列名称是否与配置文件一致，避免监听错误
        if (!MESSAGE_QUEUES_NAME.equals(rabbitmq.getQueueName())) {
            log.error("队列名称不一致，请检查配置文件");
            context.close();
            return null;
        }

        SimpleRabbitListenerContainerFactory factory = new SimpleRabbitListenerContainerFactory();
        // 最小消费者数量
        factory.setConcurrentConsumers(1);
        // 最大消费者数量
        factory.setMaxConcurrentConsumers(1);
        // 预读取一条消息
        factory.setPrefetchCount(1);
        // 手动确认消息模式
        factory.setAcknowledgeMode(AcknowledgeMode.MANUAL);
        configurer.configure(factory, connectionFactory);
        return factory;
    }

    public CachingConnectionFactory connectionFactory(
        String host,
        int port,
        String username,
        String password,
        String virtualHost
    ) {
        CachingConnectionFactory connectionFactory = new CachingConnectionFactory();
        connectionFactory.setHost(host);
        connectionFactory.setPort(port);
        connectionFactory.setUsername(username);
        connectionFactory.setPassword(password);
        connectionFactory.setVirtualHost(virtualHost);
        return connectionFactory;
    }
}
